@@ -1,6 +1,6 @@
 # ----------------------
 # Author: Andreas Alfons
-#         K.U.Leuven
+#         KU Leuven
 # ----------------------
 
 #' Cross-validation for tuning parameter selection
@@ -73,8 +73,10 @@
 #' fitting function.
 #' @param cost  a cost function measuring prediction loss.  It should expect 
 #' the observed values of the response to be passed as the first argument and 
-#' the predicted values as the second argument, and must return a non-negative 
-#' scalar value.  The default is to use the root mean squared prediction error 
+#' the predicted values as the second argument, and must return either a 
+#' non-negative scalar value, or a list with the first component containing 
+#' the prediction error and the second component containing the standard 
+#' error.  The default is to use the root mean squared prediction error 
 #' (see \code{\link{cost}}).
 #' @param K  an integer giving the number of groups into which the data should 
 #' be split (the default is five).  Keep in mind that this should be chosen 
@@ -95,6 +97,19 @@
 #' \code{\link[stats]{predict}} method of the fitted models.
 #' @param costArgs  a list of additional arguments to be passed to the 
 #' prediction loss function \code{cost}.
+#' @param selectBest  a character string specifying a criterion for selecting 
+#' the best model.  Possible values are \code{"min"} (the default) or 
+#' \code{"hastie"}.  The former selects the model with the smallest prediction 
+#' error.  The latter is useful for models with a tuning parameter controlling 
+#' the complexity of the model (e.g., penalized regression).  It selects the 
+#' most parsimonious model whose prediction error is no larger than 
+#' \code{sdFactor} standard errors above the prediction error of the best 
+#' overall model.  Note that the models are thereby assumed to be ordered 
+#' from the most parsimonious one to the most complex one.  In particular 
+#' a one-standard-error rule is frequently applied.
+#' @param sdFactor  a numeric value giving a multiplication factor of the 
+#' standard error for the selection of the best model.  This is ignored if 
+#' \code{selectBest} is \code{"min"}.
 #' @param envir  the \code{\link{environment}} in which to evaluate the 
 #' function call for fitting the models (see \code{\link{eval}}).
 #' @param seed  optional initial seed for the random number generator (see 
@@ -117,6 +132,12 @@
 #' @returnItem cv  a data frame containing the estimated prediction errors for 
 #' all combinations of tuning parameter values.  For repeated cross-validation, 
 #' those are average values over all replications.
+#' @returnItem sd  a data frame containing the estimated standard errors of the 
+#' prediction loss for all combinations of tuning parameter values.
+#' @returnItem selectBest  a character string specifying the criterion used for 
+#' selecting the best model.
+#' @returnItem sdFactor  a numeric value giving the multiplication factor of 
+#' the standard error used for the selection of the best model.
 #' @returnItem reps  a data frame containing the estimated prediction errors 
 #' from all replications for all combinations of tuning parameter values.  This 
 #' is only returned for repeated cross-validation.
@@ -129,6 +150,11 @@
 #' 
 #' @author Andreas Alfons
 #' 
+#' @references 
+#' Hastie, T., Tibshirani, R. and Friedman, J. (2009) \emph{The Elements of 
+#' Statistical Learning: Data Mining, Inference, and Prediction}.  Springer, 
+#' 2nd edition.
+#' 
 #' @seealso \code{\link{cvTool}}, \code{\link{cvFit}}, \code{\link{cvSelect}}, 
 #' \code{\link{cvFolds}}, \code{\link{cost}}
 #' 
@@ -149,6 +175,7 @@ cvTuning.function <- function(object, formula, data = NULL, x = NULL, y,
         tuning = list(), args = list(), cost = rmspe, K = 5, R = 1, 
         foldType = c("random", "consecutive", "interleaved"), folds = NULL, 
         names = NULL, predictArgs = list(), costArgs = list(), 
+        selectBest = c("min", "hastie"), sdFactor = 1, 
         envir = parent.frame(), seed = NULL, ...) {
     ## initializations
     matchedCall <- match.call()
@@ -171,7 +198,8 @@ cvTuning.function <- function(object, formula, data = NULL, x = NULL, y,
     ## call method for unevaluated function calls
     out <- cvTuning(call, data=data, x=x, y=y, tuning=tuning, cost=cost, 
         K=K, R=R, foldType=foldType, folds=folds, names=names, 
-        predictArgs=predictArgs, costArgs=costArgs, envir=envir, seed=seed)
+        predictArgs=predictArgs, costArgs=costArgs, selectBest=selectBest, 
+        sdFactor=sdFactor, envir=envir, seed=seed)
     out$call <- matchedCall
     out
 }
@@ -185,8 +213,8 @@ cvTuning.call <- function(object, data = NULL, x = NULL, y,
         tuning = list(), cost = rmspe, K = 5, R = 1, 
         foldType = c("random", "consecutive", "interleaved"), 
         folds = NULL, names = NULL, predictArgs = list(), 
-        costArgs = list(), envir = parent.frame(), 
-        seed = NULL, ...) {
+        costArgs = list(), selectBest = c("min", "hastie"), 
+        sdFactor = 1, envir = parent.frame(), seed = NULL, ...) {
     ## initializations
     matchedCall <- match.call()
     matchedCall[[1]] <- as.name("cvTuning")
@@ -199,6 +227,7 @@ cvTuning.call <- function(object, data = NULL, x = NULL, y,
         nx <- nobs(data)
     }
     if(!isTRUE(n == nx)) stop(sprintf("'%s' must have %d observations", sx, nx))
+    selectBest <- match.arg(selectBest)
     # create all combinations of tuning parameters
     tuning <- do.call("expand.grid", tuning)
     nTuning <- nrow(tuning)
@@ -238,17 +267,39 @@ cvTuning.call <- function(object, data = NULL, x = NULL, y,
                     predictArgs=predictArgs, costArgs=costArgs, envir=envir)
             })
     }
+    if(R == 1) {
+        haveList <- is.list(cv[[1]])
+        if(haveList) {
+            sd <- lapply(cv, function(x) x[[2]])
+            cv <- lapply(cv, function(x) x[[1]])
+        }
+    }
     cv <- do.call("rbind", cv)
+    if(R == 1) {
+        if(haveList) {
+            sd <- do.call("rbind", sd)
+        } else sd <- matrix(NA, nrow(cv), ncol(cv), dimnames=dimnames(cv))
+        sd <- data.frame(Fit=seq_len(nTuning), sd, row.names=NULL)
+    }
     cv <- data.frame(Fit=rep(seq_len(nTuning), each=R), cv, row.names=NULL)
     ## compute average results in case of repeated CV
     if(R > 1) {
         reps <- cv
         cv <- aggregate(reps[, -1, drop=FALSE], reps[, 1, drop=FALSE], mean)
+        sd <- aggregate(reps[, -1, drop=FALSE], reps[, 1, drop=FALSE], sd)
     }
     ## find optimal values of tuning parameters
-    best <- sapply(cv[, -1, drop=FALSE], which.min)
+    if(selectBest == "min") {
+        sdFactor <- NA
+        best <- sapply(cv[, -1, drop=FALSE], selectMin)
+    } else {
+        sdFactor <- rep(sdFactor, length.out=1)
+        best <- sapply(names(cv)[-1], 
+            function(j) selectHastie(cv[, j], sd[, j], sdFactor=sdFactor))
+    }
     ## construct return object
-    out <- list(n=folds$n, K=folds$K, R=R, tuning=tuning, best=best, cv=cv)
+    out <- list(n=folds$n, K=folds$K, R=R, tuning=tuning, best=best, 
+        cv=cv, sd=sd, selectBest=selectBest, sdFactor=sdFactor)
     if(R > 1) out$reps <- reps
     out$seed <- seed
     out$call <- matchedCall
